@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -189,7 +189,7 @@ void pool_discard_cp(char *user, char *database, int protoMajor)
 
 	if (p == NULL)
 	{
-		pool_error("pool_discard_cp: cannot get connection pool for user %s datbase %s", user, database);
+		pool_error("pool_discard_cp: cannot get connection pool for user %s database %s", user, database);
 		return;
 	}
 
@@ -332,7 +332,8 @@ void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend)
 
 	/* no other timer found. set my timer */
 	pool_debug("pool_connection_pool_timer: set alarm after %d seconds", pool_config->connection_life_time);
-	pool_alarm(pool_backend_timer_handler, pool_config->connection_life_time);
+	pool_signal(SIGALRM, pool_backend_timer_handler);
+	alarm(pool_config->connection_life_time);
 }
 
 /*
@@ -418,7 +419,8 @@ void pool_backend_timer(void)
 		nearest = pool_config->connection_life_time - (now - nearest);
 		if (nearest <= 0)
 		  nearest = 1;
-		pool_alarm(pool_backend_timer_handler, nearest);
+		pool_signal(SIGALRM, pool_backend_timer_handler);
+		alarm(nearest);
 	}
 
 	POOL_SETMASK(&UnBlockSig);
@@ -658,12 +660,27 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 			}
 			else		/* select returns error */
 			{
-				if((errno == EINTR && retry) || errno == EAGAIN)
+				if ((errno == EINTR && retry) || errno == EAGAIN)
 				{
 					pool_log("connect_inet_domain_socket: select() interrupted. retrying...");
 					continue;
 				}
-				pool_log("connect_inet_domain_socket: select() interrupted");
+
+				/*
+				 * select(2) was interrupted by certain signal and we guess it
+				 * was not SIGALRM because health_check_timer_expired was not
+				 * set (if the variable was set, we can assume that SIGALRM
+				 * handler was called). Surely this is not a health check time
+				 * out. We can assume that this is a transient case. So we
+				 * will retry again...
+				 */
+				if (health_check_timer_expired == 0 && errno == EINTR)
+				{
+					pool_log("connect_inet_domain_socket: select() interrupted by certain signal. retrying...");
+					continue;
+				}
+
+				pool_log("connect_inet_domain_socket: select() interrupted or returns error by \"%s\"", strerror(errno));
 				close(fd);
 				return -1;
 			}
@@ -705,7 +722,7 @@ static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int s
 }
 
 /*
- * Create actual connections to backends.
+ * create actual connections to backends
  */
 static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 {
@@ -737,7 +754,7 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 			pool_error("new_connection: create_cp() failed");
 
 			/* If fail_over_on_backend_error is true, do failover.
-			 * Otherwise, just exit this session or skip next health node.
+			 * Otherwise, just exit this session.
 			 */
 			if (pool_config->fail_over_on_backend_error)
 			{
@@ -745,27 +762,7 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 			}
 			else
 			{
-				/*
-				 * If we are in streaming replication mode and the node is a
-				 * standby node, then we skip this node to avoid fail over.
-				 */
-				if (STREAM && !IS_PRIMARY_NODE_ID(i))
-				{
-					pool_log("failed to create a backend %d connection. Skip this backend because because fail_over_on_backend_error is off and we are in streaming replication mode and node is standby node", i);
-
-					/* set down status to local status area */
-					*(my_backend_status[i]) = CON_DOWN;
-
-					/* make sure that we need to restart the process after
-					 * finishing this session
-					 */
-					pool_get_my_process_info()->need_to_restart = 1;
-					continue;
-				}
-				else
-				{
-					pool_log("new_connection: do not failover because fail_over_on_backend_error is off");
-				}
+				pool_log("new_connection: do not failover because fail_over_on_backend_error is off");
 			}
 			child_exit(1);
 		}
