@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -39,6 +39,7 @@
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/wait.h>
 
 #include "pool.h"
 #include "pool_config.h"
@@ -77,7 +78,7 @@ static int wd_send_node_packet(WD_PACKET_NO packet_no, int *node_id_set, int cou
 static int wd_chk_node_mask (WD_PACKET_NO packet_no, int *node_id_set, int count);
 
 void wd_calc_hash(const char *str, int len, char *buf);
-int wd_packet_to_string(WdPacket pkt, char *str, int maxlen);
+int wd_packet_to_string(WdPacket *pkt, char *str, int maxlen);
 
 static void * wd_thread_negotiation(void * arg);
 static int send_packet_for_all(WdPacket *packet);
@@ -199,11 +200,13 @@ wd_create_send_socket(char * hostname, int port)
 	if ( setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *) &one, sizeof(one)) == -1 )
 	{
 		pool_error("wd_create_send_socket: setsockopt(TCP_NODELAY) failed. reason: %s", strerror(errno));
+		close(sock);
 		return -1;
 	}
 	if ( setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(one)) == -1 )
 	{
 		pool_error("wd_create_send_socket: setsockopt(SO_KEEPALIVE) failed. reason: %s", strerror(errno));
+		close(sock);
 		return -1;
 	}
 
@@ -484,8 +487,7 @@ wd_recv_packet(int sock, WdPacket * recv_pack)
 			if (read_size == len)
 			{
 
-				if ((ntohl(buf.packet_no) >= WD_INVALID) &&
-					(ntohl(buf.packet_no) <= WD_READY ))
+				if (ntohl(buf.packet_no) <= WD_READY)
 				{
 					ntoh_wd_packet(recv_pack,&buf);
 				}
@@ -529,8 +531,11 @@ wd_thread_negotiation(void * arg)
 	if (strlen(pool_config->wd_authkey))
 	{
 		/* calculate hash from packet */
-		pack_str_len = wd_packet_to_string(*(thread_arg->packet), pack_str, sizeof(pack_str));
+		pack_str_len = wd_packet_to_string(thread_arg->packet, pack_str, sizeof(pack_str));
 		wd_calc_hash(pack_str, pack_str_len, thread_arg->packet->hash);
+
+		if ((thread_arg->packet->hash)[0] == '\0')
+			pool_log("wd_thread_negotiasion: failed to calculate wd_authkey hash from a send packet");
 	}
 
 	/* packet send to target watchdog */
@@ -654,7 +659,7 @@ send_packet_4_nodes(WdPacket *packet, WD_SEND_TYPE type)
 	while (p->status != WD_END)
 	{
 		/* don't send packet to pgpool in down */
-		if (p->status == WD_DOWN || 
+		if (p->status == WD_DOWN ||
 		    (packet->packet_no != WD_ADD_REQ && p->status == WD_INIT))
 		{
 			p->is_contactable = false;
@@ -939,7 +944,8 @@ ntoh_wd_lock_packet(WdPacket * to, WdPacket * from)
 int
 wd_escalation(void)
 {
-	int rtn;
+	int rtn, r;
+	bool has_error = false;
 
 	pool_log("wd_escalation: escalating to master pgpool");
 
@@ -954,12 +960,31 @@ wd_escalation(void)
 	/* execute escalation command */
 	if (strlen(pool_config->wd_escalation_command))
 	{
-		system(pool_config->wd_escalation_command);
+		r = system(pool_config->wd_escalation_command);
+		if (WIFEXITED(r))
+		{
+			if (WEXITSTATUS(r) == EXIT_SUCCESS)
+				pool_log("wd_escalation: escalation command succeeded");
+			else
+			{
+				pool_error("wd_escalation: escalation command failed. exit status: %d", WEXITSTATUS(r));
+				has_error = true;
+			}
+		}
+		else
+		{
+			pool_error("wd_escalation: escalation command exit abnormally");
+			has_error = true;
+		}
 	}
 
 	/* interface up as delegate IP */
 	if (strlen(pool_config->delegate_IP) != 0)
-		wd_IP_up();
+	{
+		r = wd_IP_up();
+		if (r == WD_NG)
+			has_error = true;
+	}
 
 	/* set master status to the wd list */
 	wd_set_wd_list(pool_config->wd_hostname, pool_config->port,
@@ -970,7 +995,10 @@ wd_escalation(void)
 	rtn = wd_declare();
 	if (rtn == WD_OK)
 	{
-		pool_log("wd_escalation: escalated to master pgpool successfully");
+		if (has_error)
+			pool_log("wd_escalation: escalated to master pgpool with some errors");
+		else
+			pool_log("wd_escalation: escalated to master pgpool successfully");
 	}
 
 	return rtn;
@@ -1133,26 +1161,38 @@ wd_calc_hash(const char *str, int len, char *buf)
 	/* use first half of authkey as username, last half as password */
 	authkey_len = strlen(pool_config->wd_authkey);
 
+	if (len <= 0 || authkey_len <= 0)
+		goto wd_calc_hash_error;
+
 	username_len = authkey_len / 2;
 	pass_len = authkey_len - username_len;
-	snprintf(username, username_len + 1, pool_config->wd_authkey);
-	snprintf(pass, pass_len + 1, pool_config->wd_authkey + username_len);
+	if ( snprintf(username, username_len + 1, "%s", pool_config->wd_authkey) < 0
+	  || snprintf(pass, pass_len + 1, "%s", pool_config->wd_authkey + username_len) < 0)
+		goto wd_calc_hash_error;
 
 	/* calculate hash using md5 encrypt */
-	pool_md5_encrypt(pass, username, strlen(username), buf + MD5_PASSWD_LEN + 1);
+	if (! pool_md5_encrypt(pass, username, strlen(username), buf + MD5_PASSWD_LEN + 1))
+		goto wd_calc_hash_error;
 	buf[(MD5_PASSWD_LEN+1)*2-1] = '\0';
 
-	pool_md5_encrypt(buf+MD5_PASSWD_LEN+1, str, len, buf);
+	if (! pool_md5_encrypt(buf+MD5_PASSWD_LEN+1, str, len, buf))
+		goto wd_calc_hash_error;
 	buf[MD5_PASSWD_LEN] = '\0';
+
+	return;
+
+wd_calc_hash_error:
+	buf[0] = '\0';
+	return;
 }
 
 int
-wd_packet_to_string(WdPacket pkt, char *str, int maxlen)
+wd_packet_to_string(WdPacket *pkt, char *str, int maxlen)
 {
 	int len;
 
 	len = snprintf(str, maxlen, "no=%d tv_sec=%ld tv_usec=%ld",
-	               pkt.packet_no, pkt.send_time.tv_sec, pkt.send_time.tv_usec);
+	               pkt->packet_no, pkt->send_time.tv_sec, pkt->send_time.tv_usec);
 
 	return len;
 }
